@@ -11,12 +11,12 @@ from pikepdf import (
     Array,
     ContentStreamInlineImage,
     Dictionary,
+    Matrix,
     Name,
     Object,
     Operator,
     Page,
     Pdf,
-    PdfMatrix,
     Rectangle,
     parse_content_stream,
 )
@@ -26,12 +26,14 @@ from pikepdf import (
 
 @pytest.fixture
 def graph(resources):
-    yield Pdf.open(resources / 'graph.pdf')
+    with Pdf.open(resources / 'graph.pdf') as pdf:
+        yield pdf
 
 
 @pytest.fixture
 def fourpages(resources):
-    yield Pdf.open(resources / 'fourpages.pdf')
+    with Pdf.open(resources / 'fourpages.pdf') as pdf:
+        yield pdf
 
 
 @pytest.fixture
@@ -41,20 +43,30 @@ def graph_page(graph):
 
 def test_page_boxes(graph_page):
     page = graph_page
-    assert page.mediabox == page.cropbox == page.trimbox
-    page.cropbox = [0, 0, page.mediabox[2] - 100, page.mediabox[3] - 100]
+    assert page.mediabox == page.cropbox == page.trimbox == page.artbox == page.bleedbox
     page.mediabox = [
-        page.mediabox[0] - 50,
-        page.mediabox[1] - 50,
-        page.mediabox[2] + 50,
-        page.mediabox[3] + 50,
+        page.mediabox[0],
+        page.mediabox[1],
+        page.mediabox[2],
+        page.mediabox[3],
     ]
-    page.trimbox = [50, 50, page.mediabox[2] - 50, page.mediabox[3] - 50]
+    inset = 10
+    page.bleedbox = [inset, inset, page.mediabox[2] - inset, page.mediabox[3] - inset]
+    inset = 20
+    page.trimbox = [inset, inset, page.mediabox[2] - inset, page.mediabox[3] - inset]
+    inset = 30
+    page.cropbox = Array(
+        [inset, inset, page.mediabox[2] - inset, page.mediabox[3] - inset]
+    )
+    inset = 40
+    page.artbox = Rectangle(
+        inset, inset, page.mediabox[2] - inset, page.mediabox[3] - inset
+    )
 
+    assert page.mediabox != page.bleedbox
+    assert page.mediabox != page.artbox
     assert page.mediabox != page.cropbox
-    assert page.cropbox != page.mediabox
-
-    page.cropbox = Array([0, 0, 50, 50])
+    assert page.mediabox != page.trimbox
 
 
 def test_invalid_boxes(graph_page):
@@ -122,25 +134,24 @@ class TestAddResource:
 def test_add_unowned_page():  # issue 174
     pdf = Pdf.new()
     d = Dictionary(Type=Name.Page)
-    pdf.pages.append(d)
+    pdf.pages.append(Page(d))
 
 
 def test_failed_add_page_cleanup():
     pdf = Pdf.new()
     d = Dictionary(Type=Name.NotAPage)
     num_objects = len(pdf.objects)
-    with pytest.raises(TypeError, match="only pages can be inserted"):
+    with pytest.raises(TypeError, match="pikepdf.Page"):
         pdf.pages.append(d)
     assert len(pdf.pages) == 0
 
-    # If we fail to add a new page, we expect one new null object handle to be
-    # be added (since QPDF does not remove the object outright)
-    assert len(pdf.objects) == num_objects + 1, "QPDF semantics changed"
-    assert pdf.objects[-1] is None, "Left a stale object behind without deleting"
+    # If we fail to add a new page, confirm we did not create a new object
+    assert len(pdf.objects) == num_objects, "A dangling page object was created"
+    assert pdf.objects[-1] is not None, "Left a stale object behind without deleting"
 
     # But we'd better not delete an existing object...
     d2 = pdf.make_indirect(Dictionary(Type=Name.StillNotAPage))
-    with pytest.raises(TypeError, match="only pages can be inserted"):
+    with pytest.raises(TypeError, match="pikepdf.Page"):
         pdf.pages.append(d2)
     assert len(pdf.pages) == 0
 
@@ -159,6 +170,9 @@ def test_formx(graph, outpdf):
     new_page.obj.Contents = graph.make_stream(cs)
     graph.save(outpdf)
 
+    assert formx_placed_name in new_page.form_xobjects
+    assert new_page.form_xobjects[formx_placed_name] == formx
+
 
 def test_fourpages_to_4up(fourpages, graph, outpdf):
     pdf = Pdf.new()
@@ -184,8 +198,8 @@ def test_fourpages_to_4up(fourpages, graph, outpdf):
 
 
 def _simple_interpret_content_stream(page: Page | Object):
-    ctm = PdfMatrix.identity()
-    stack: list[PdfMatrix] = []
+    ctm = Matrix()
+    stack: list[Matrix] = []
     for instruction in parse_content_stream(page, operators='q Q cm Do'):
         if isinstance(instruction, ContentStreamInlineImage):
             continue
@@ -195,7 +209,7 @@ def _simple_interpret_content_stream(page: Page | Object):
         elif op == Operator('Q'):
             ctm = stack.pop()
         elif op == Operator('cm'):
-            ctm = PdfMatrix(operands) @ ctm
+            ctm = Matrix(operands) @ ctm
         elif op == Operator('Do'):
             xobj_name = operands[0]
             yield (xobj_name, ctm)
@@ -279,7 +293,7 @@ def test_page_attrs(graph):
 
     del graph.pages[0].Resources
     with pytest.raises(
-        AttributeError, match=r"can't delete|property of 'Page' object has no deleter"
+        AttributeError, match=r"can't delete|property( '')? of 'Page' object has no deleter"
     ):
         del graph.pages[0].obj
     del graph.pages[0]['/Contents']
